@@ -1,12 +1,8 @@
 package org.gcube.applicationsupportlayer.social;
 
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.IOException;
 import java.text.SimpleDateFormat;
 import java.util.Date;
 import java.util.List;
-import java.util.Properties;
 import java.util.UUID;
 
 import org.gcube.application.framework.core.session.ASLSession;
@@ -14,7 +10,7 @@ import org.gcube.applicationsupportlayer.social.mailing.EmailPlugin;
 import org.gcube.common.homelibrary.home.exceptions.InternalErrorException;
 import org.gcube.common.homelibrary.home.workspace.WorkspaceItem;
 import org.gcube.common.homelibrary.home.workspace.WorkspaceSharedFolder;
-import org.gcube.portal.custom.communitymanager.OrganizationsUtil;
+import org.gcube.common.portal.PortalContext;
 import org.gcube.portal.databook.shared.ApplicationProfile;
 import org.gcube.portal.databook.shared.Notification;
 import org.gcube.portal.databook.shared.NotificationChannelType;
@@ -38,9 +34,6 @@ import org.slf4j.LoggerFactory;
 public class ApplicationNotificationsManager extends SocialPortalBridge implements NotificationsManager {
 	private static final Logger _log = LoggerFactory.getLogger(ApplicationNotificationsManager.class);
 
-	private static final String SENDER_EMAIL = "notificationSenderEmail";
-	private static final String GATEWAY_NAME = "portalinstancename";
-
 	private String portalName;
 	private String senderEmail;
 	/**
@@ -49,8 +42,8 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 	 */
 	public ApplicationNotificationsManager(ASLSession session) {
 		super(session);
-		portalName = getPortalInstanceName();
-		senderEmail = getNotificationSenderEmail();
+		portalName = PortalContext.getPortalInstanceName();
+		senderEmail = PortalContext.getConfiguration().getSenderEmail();
 		_log.warn("Asked for Simple Notification (without redirect to creator)");
 	}
 	/**
@@ -64,15 +57,15 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 	 */
 	public ApplicationNotificationsManager(ASLSession session, String portletClassName) {
 		super(session, portletClassName);
-		portalName = getPortalInstanceName();
-		senderEmail = getNotificationSenderEmail();
+		portalName = PortalContext.getPortalInstanceName();
+		senderEmail = PortalContext.getConfiguration().getSenderEmail();
 	}
 	/**
 	 * actually save the notification to the store
 	 * @param notification2Save the notification instance to save
 	 * @return true if the notification was sent ok
 	 */
-	private boolean saveNotification(Notification notification2Save) {
+	private boolean saveNotification(Notification notification2Save, String ... hashtags) {
 		_log.trace("Trying to send notification to: " + notification2Save.getUserid() +  " Type: " + notification2Save.getType());
 		if (notification2Save.getSenderid().compareTo(notification2Save.getUserid()) == 0) {
 			_log.trace("Sender and Receiver are the same " + notification2Save.getUserid() +  " Notification Stopped");
@@ -93,15 +86,15 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 				_log.error("Error While trying to save Notification");
 		}
 		if (channels.contains(NotificationChannelType.EMAIL))
-			EmailPlugin.getInstance().sendNotification(notification2Save, aslSession.getGroupName(), portalName, senderEmail);
+			EmailPlugin.getInstance().sendNotification(notification2Save, aslSession.getGroupName(), portalName, senderEmail, hashtags);
 
 		if (channels.isEmpty()) {
 			_log.info("Notification was not needed as "+ notification2Save.getUserid()  +" decided not to be notified for " + notification2Save.getType());
 			result = true;
 		}
 		return result;
-
 	}
+
 	/**
 	 * return the url of the application if exists in the profile
 	 * @return .
@@ -112,8 +105,8 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 			return applicationProfile.getUrl();			
 		}
 		else {
-			_log.warn("applicationProfile NULL or url is empty");
-			return "";
+			_log.warn("applicationProfile NULL or url is empty returning " + "/group/data-e-infrastructure-gateway");
+			return "/group/data-e-infrastructure-gateway";
 		}
 	}
 	/**
@@ -208,7 +201,7 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 
 		return saveNotification(not);
 	}
-	
+
 	/**
 	 * {@inheritDoc}
 	 * @throws InternalErrorException 
@@ -317,22 +310,22 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 	public boolean notifyAddedItem(String userIdToNotify, WorkspaceItem item, WorkspaceSharedFolder sharedFolder) throws InternalErrorException {
 		String notifyText = sharedFolder.isVreFolder() ? 
 				"added "+ item.getName() +" to the workspace group folder " + sharedFolder.getDisplayName() :
-			"added "+ item.getName() +" to your workspace shared folder "+ item.getPath().substring(0,item.getPath().lastIndexOf('/'));
-		
-		Notification not = new Notification(
-				UUID.randomUUID().toString(), 
-				NotificationType.WP_ITEM_NEW, 
-				userIdToNotify, //user no notify
-				item.getId(),  //the 
-				new Date(),
-				getApplicationUrl()+"?itemid="+item.getParent().getId(), 
-				notifyText,
-				false, 
-				aslSession.getUsername(),
-				aslSession.getUserFullName(), 
-				aslSession.getUserAvatarId());
+					"added "+ item.getName() +" to your workspace shared folder "+ item.getPath().substring(0,item.getPath().lastIndexOf('/'));
 
-		return saveNotification(not);
+				Notification not = new Notification(
+						UUID.randomUUID().toString(), 
+						NotificationType.WP_ITEM_NEW, 
+						userIdToNotify, //user no notify
+						item.getId(),  //the 
+						new Date(),
+						getApplicationUrl()+"?itemid="+item.getParent().getId(), 
+						notifyText,
+						false, 
+						aslSession.getUsername(),
+						aslSession.getUserFullName(), 
+						aslSession.getUserAvatarId());
+
+				return saveNotification(not);
 	}
 	/**
 	 * {@inheritDoc}
@@ -342,7 +335,7 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 	public boolean notifyMovedItem(String userIdToNotify, WorkspaceItem item, WorkspaceSharedFolder sharedFolder) throws InternalErrorException {
 		String notifyText = sharedFolder.isVreFolder() ? "removed item  "+ item.getName() +" from the workspace group folder " + sharedFolder.getDisplayName():
 			"removed item "+ item.getName() +" from your workspace shared folder \""+ sharedFolder.getName()+"\"";
-		
+
 		Notification not = new Notification(
 				UUID.randomUUID().toString(), 
 				NotificationType.WP_ITEM_DELETE, 
@@ -434,33 +427,47 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 	 * {@inheritDoc}
 	 */
 	@Override
-	public boolean notifyMessageReceived(String userIdToNotify,	String subject) {
+	public boolean notifyMessageReceived(String userIdToNotify,	String messageId, String subject, String messageText,  String ... otherRecipientsFullNames) {
+		String[] optionalParams = {subject};
+		String otherRecipientNames = "";
+		if (otherRecipientsFullNames != null && otherRecipientsFullNames.length > 0) {
+			otherRecipientNames = "<br/><div> - This message was also sent to: <ul>";
+			for (int i = 0; i < otherRecipientsFullNames.length; i++) {
+				otherRecipientNames += "<li> " + otherRecipientsFullNames[i] + " </li>";
+			}
+			otherRecipientNames += "</ul></div><div>If you reply, your message will be also delivered to them.</div>";
+		}
+		String attachmentsNotice = "<div>Please note that email replies do not support attachments.</div>";
+		
 		Notification not = new Notification(
 				UUID.randomUUID().toString(), 
 				NotificationType.MESSAGE, 
 				userIdToNotify, //user no notify
-				"messageid_not_provided",  //the 
+				messageId,  //the unique identifier of the message
 				new Date(),
 				"/group/data-e-infrastructure-gateway/messages", 
-				"sent you a message with subject: " + escapeHtml(subject),
+				"sent you a message: " 
+						+ "<br/><div style=\"margin-top: 10px;  margin-bottom: 10px;  margin-left: 50px;  padding-left: 15px;  border-left: 3px solid #ccc; font-style: italic;\">"
+						+ messageText +"</div>"
+								+ otherRecipientNames + attachmentsNotice,
 				false, 
 				aslSession.getUsername(),
 				aslSession.getUserFullName(), 
 				aslSession.getUserAvatarId());
 
-		return saveNotification(not);
+		return saveNotification(not, optionalParams);
 	}
 	/**
 	 * {@inheritDoc}
 	 */
 	@Override
-	public boolean notifyPost(String userIdToNotify, String feedid,	String feedText) {
+	public boolean notifyPost(String userIdToNotify, String feedid,	String feedText, String ... hashtags) {
 
 		StringBuilder notificationText = new StringBuilder();
-		notificationText.append("posted a news on <b>").append(aslSession.getGroupName()).append(":</b>") // has done something
-		.append("<br /><br /> ").append(escapeHtml(feedText)).append(". ")
-		.append("<br /><br />Follow the discussion and comment it. ");
-
+		notificationText.append("posted a news on <b> ").append(aslSession.getGroupName()).append(":</b>") // has done something
+		.append("<br /><br /> ").append(feedText).append(" ")
+		.append("<br /><br />");
+		
 		Notification not = new Notification(
 				UUID.randomUUID().toString(), 
 				NotificationType.POST_ALERT, 
@@ -473,7 +480,7 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 				aslSession.getUsername(),
 				aslSession.getUserFullName(), 
 				aslSession.getUserAvatarId());
-		return saveNotification(not);
+		return saveNotification(not, hashtags);
 	}
 	/**
 	 * {@inheritDoc}
@@ -486,12 +493,15 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 				userIdToNotify, //user no notify
 				feedid,  //the post 
 				new Date(),
-				"/group/data-e-infrastructure-gateway?oid="+feedid, 
-				"commented on your post: " + escapeHtml(feedText),
-				false, 
-				aslSession.getUsername(),
-				aslSession.getUserFullName(), 
-				aslSession.getUserAvatarId());
+				getApplicationUrl()+"?oid="+feedid, 
+				"commented on your post: " 
+						+ "<div style=\"margin-top: 10px;  margin-bottom: 10px;  margin-left: 50px;  padding-left: 15px;  "
+						+ "border-left: 3px solid #ccc; font-style: italic;\">\""
+						+ feedText +"\"</div>",
+						false, 
+						aslSession.getUsername(),
+						aslSession.getUserFullName(), 
+						aslSession.getUserAvatarId());
 		return saveNotification(not);
 	}
 	/**
@@ -500,22 +510,26 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 	@Override
 	public boolean notifyCommentReply(String userIdToNotify, String feedid, String commentText, String feedOwnerFullName, String feedOwnerId) {
 		String notificationText =  (aslSession.getUsername().compareTo(feedOwnerId) == 0) ? 
-				"also commented on his post: " + escapeHtml(commentText)
+				"also commented on his post: "  
+				+ "<div style=\"margin-top: 10px;  margin-bottom: 10px;  margin-left: 50px;  padding-left: 15px;  border-left: 3px solid #ccc; font-style: italic;\">\""
+				+ commentText  +"\"</div>"
 				: 
-					"also commented on " + feedOwnerFullName + "'s post: " + escapeHtml(commentText);
-				Notification not = new Notification(
-						UUID.randomUUID().toString(), 
-						NotificationType.COMMENT, 
-						userIdToNotify, //user no notify
-						feedid,  //the post 
-						new Date(),
-						"/group/data-e-infrastructure-gateway?oid="+feedid, 
-						notificationText,
-						false, 
-						aslSession.getUsername(),
-						aslSession.getUserFullName(), 
-						aslSession.getUserAvatarId());
-				return saveNotification(not);
+					"also commented on " + feedOwnerFullName + "'s post: " 
+					+ "<div style=\"margin-top: 10px;  margin-bottom: 10px;  margin-left: 50px;  padding-left: 15px;  border-left: 3px solid #ccc; font-style: italic;\">\""
+					+ commentText +"\"</div>";
+		Notification not = new Notification(
+				UUID.randomUUID().toString(), 
+				NotificationType.COMMENT, 
+				userIdToNotify, //user no notify
+				feedid,  //the post 
+				new Date(),
+				getApplicationUrl()+"?oid="+feedid, 
+				notificationText,
+				false, 
+				aslSession.getUsername(),
+				aslSession.getUserFullName(), 
+				aslSession.getUserAvatarId());
+		return saveNotification(not);
 	}
 	/**
 	 * {@inheritDoc}
@@ -528,12 +542,14 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 				userIdToNotify, //user no notify
 				feedid,  //the post 
 				new Date(),
-				"/group/data-e-infrastructure-gateway?oid="+feedid, 
-				"commented on one of your favorite posts: " + escapeHtml(commentText),
-				false, 
-				aslSession.getUsername(),
-				aslSession.getUserFullName(), 
-				aslSession.getUserAvatarId());
+				getApplicationUrl()+"?oid="+feedid, 
+				"commented on one of your favorite posts: " 
+						+ "<div style=\"margin-top: 10px;  margin-bottom: 10px;  margin-left: 50px;  padding-left: 15px;  border-left: 3px solid #ccc; font-style: italic;\">\""
+						+ commentText +"\"</div>",
+						false, 
+						aslSession.getUsername(),
+						aslSession.getUserFullName(), 
+						aslSession.getUserAvatarId());
 		return saveNotification(not);
 	}
 	/**
@@ -547,8 +563,9 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 				userIdToNotify, //user no notify
 				feedid,  //the post 
 				new Date(),
-				"/group/data-e-infrastructure-gateway?oid="+feedid, 
-				"mentioned you: " + escapeHtml(feedText),
+				getApplicationUrl()+"?oid="+feedid, 
+				"mentioned you: " + "<div style=\"margin-top: 10px;  margin-bottom: 10px;  margin-left: 50px;  padding-left: 15px;  border-left: 3px solid #ccc; font-style: italic;\">\""
+						+ feedText +"\"</div>",
 				false, 
 				aslSession.getUsername(),
 				aslSession.getUserFullName(), 
@@ -560,14 +577,15 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 	 */
 	@Override
 	public boolean notifyLikedFeed(String userIdToNotify, String feedid, String feedText) {
-		Notification not = new Notification(
+		
+	Notification not = new Notification(
 				UUID.randomUUID().toString(), 
 				NotificationType.LIKE, 
 				userIdToNotify, //user no notify
 				feedid,  //the post 
 				new Date(),
-				"/group/data-e-infrastructure-gateway?oid="+feedid, 
-				"favorited your post: " + escapeHtml(feedText),
+				getApplicationUrl()+"?oid="+feedid, 
+				"favorited/subscribed to one of your post",
 				false, 
 				aslSession.getUsername(),
 				aslSession.getUserFullName(), 
@@ -778,7 +796,6 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 
 		return saveNotification(not);
 	}
-
 	/**
 	 * {@inheritDoc}
 	 */
@@ -811,57 +828,56 @@ public class ApplicationNotificationsManager extends SocialPortalBridge implemen
 				aslSession.getUserAvatarId());
 
 		return saveNotification(not);
-	}	
-
-	/**
-	 * read the portal instance name from a property file and returns it
-	 */
-	private static String getPortalInstanceName() {
-		//get the portles to look for from the property file
-		Properties props = new Properties();
-		String toReturn = "";
-
-		try {
-			String propertyfile = OrganizationsUtil.getTomcatFolder()+"conf/gcube-data.properties";			
-			File propsFile = new File(propertyfile);
-			FileInputStream fis = new FileInputStream(propsFile);
-			props.load( fis);
-			toReturn = props.getProperty(GATEWAY_NAME);
-		}
-		//catch exception in case properties file does not exist
-		catch(IOException e) {
-			toReturn = "D4science Gateway";
-			_log.error("gcube-data.properties file not found under $CATALINA_HOME/conf dir, returning default Portal Name " + toReturn);
-			return toReturn;
-		}
-		_log.debug("Returning Gateway Name: " + toReturn );
-		return toReturn;
 	}
-
 	/**
-	 * read the sender email for notifications name from a property file and returns it
+	 * {@inheritDoc}
 	 */
-	private static String getNotificationSenderEmail() {
-		//get the portles to look for from the property file
-		Properties props = new Properties();
-		String toReturn = "";
+	@Override
+	public boolean notifyTDMTabularResourceSharing(String userIdToNotify, String tabularResourceName, String encodedTabularResourceParams) throws Exception {
+		Notification not = new Notification(
+				UUID.randomUUID().toString(), 
+				NotificationType.TDM_TAB_RESOURCE_SHARE, 
+				userIdToNotify, //user no notify
+				tabularResourceName,  
+				new Date(),
+				getApplicationUrl()+"?"+encodedTabularResourceParams, 
+				"shared the Tabular Resource \""+ tabularResourceName +"\" with you",
+				false, 
+				aslSession.getUsername(),
+				aslSession.getUserFullName(), 
+				aslSession.getUserAvatarId());
 
-		try {
-			String propertyfile = OrganizationsUtil.getTomcatFolder()+"conf/gcube-data.properties";			
-			File propsFile = new File(propertyfile);
-			FileInputStream fis = new FileInputStream(propsFile);
-			props.load( fis);
-			toReturn = props.getProperty(SENDER_EMAIL);
-		}
-		//catch exception in case properties file does not exist
-		catch(IOException e) {
-			toReturn = "do-not-reply@d4science.org";
-			_log.error("gcube-data.properties file not found under $CATALINA_HOME/conf dir, returning default Email" + toReturn);
-			return toReturn;
-		}
-		_log.debug("Returning SENDER_EMAIL: " + toReturn );
-		return toReturn;
+		return saveNotification(not);
 	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
+	public boolean notifyTDMObjectSharing(String userIdToNotify, NotificationType type, String tdmObjectName, String encodedTabularResourceParams) throws Exception {
+		if  (! (type == NotificationType.TDM_RULE_SHARE || type ==  NotificationType.TDM_TEMPLATE_SHARE))
+			throw new IllegalArgumentException("Type must be either TDM_RULE_SHARE or TDM_TEMPLATE_SHARE");
 
+		String notificationText = "shared the Tabular Data Manager";
+		notificationText += (type == NotificationType.TDM_RULE_SHARE) ?  " Rule " : " Template ";
+		notificationText += "\"" + tdmObjectName + "\" with you";
 
+		String url = getApplicationUrl();
+		if (encodedTabularResourceParams != null && encodedTabularResourceParams.compareTo("") != 0)
+			url += "?"+encodedTabularResourceParams;
+
+		Notification not = new Notification(
+				UUID.randomUUID().toString(), 
+				type, 
+				userIdToNotify, //user no notify
+				tdmObjectName,  
+				new Date(),
+				url, 
+				notificationText,
+				false, 
+				aslSession.getUsername(),
+				aslSession.getUserFullName(), 
+				aslSession.getUserAvatarId());
+
+		return saveNotification(not);
+	}
 }
